@@ -1207,6 +1207,10 @@ class Database:
 
             json_content = json.loads(response.content)
             json_content = json_content.replace(' nan', '\" nan\"')
+            json_content = json_content.replace('N/A\\n\\n', '0')
+            json_content = json_content.replace('N/A\\n', '0')
+            json_content = json_content.replace('0\\n', '0')
+
 
             result = ast.literal_eval(json_content)
 
@@ -1397,6 +1401,56 @@ class Database:
                 if check_bool_dict[error] == False:
                     err_msg += error + ','
             print(err_msg)
+
+    def get_hk_cbbc(self, code, start_date, end_date):
+
+        check_bool_dict = self.check_hk_cbbc(code, start_date, end_date)
+
+        if False not in list(check_bool_dict.values()):
+            link_url = 'http://www.hkfdb.net/data_api?'
+            token_str = 'token=' + str(self.authToken)
+            code_str = 'code=' + code
+            database_str = 'database=' + 'hk_cbbc'
+            start_date_str = 'start_date=' + str(start_date)
+            end_date_str = 'end_date=' + str(end_date)
+            link_str = link_url + '&' + token_str + '&' + database_str \
+                       + '&' + start_date_str + '&' + end_date_str + '&' + code_str
+
+            response = requests.get(link_str)
+            response_ok = response_check(response)
+            if response_ok == True:
+
+                result_list = json.loads(json.loads(response.content).replace("'", "\""))
+
+                columns = ['cbbc_code', 'cbbc_name', 'date', 'num_of_cbbc_bought', 'avg_price_per_cbbc_bought',
+                           'num_of_cbbc_sold', 'avg_price_per_cbbc_sold', 'num_of_cbbc_in_mkt', 'pct_of_issued_in_mkt',
+                           'total_issued_size', 'currency', 'day_high', 'day_low', 'close_price', 'volume', 'turnover',
+                           'issuer', 'underlying', 'bull_bear', 'cbbc_type', 'cbbc_category', 'listing_date',
+                           'last_trading_date', 'maturity_date', 'mce', 'strike_or_call_currency', 'strike_level',
+                           'call_level', 'entitlement_ratio', 'delisting_date']
+
+                df_list = []
+                for result in result_list:
+                    for item, content in result.items():
+                        if type(content) is list:
+                            df = pd.DataFrame(content, columns=columns)
+                            df['date'] = df['date'] / 1000000000
+                            df['date'] = pd.to_datetime(df['date'], unit='s')
+                            df_list.append(df)
+
+                df = pd.concat(df_list)
+                df = df.sort_values('date')
+                df = df.reset_index()
+                df = df.drop('index', axis=1)
+
+                return df
+
+        else:
+            err_msg = 'Error in: '
+            for error in check_bool_dict:
+                if check_bool_dict[error] == False:
+                    err_msg += error + ','
+            print(err_msg)
             
     #########################################################################################
 
@@ -1901,6 +1955,39 @@ class Database:
 
         return check_bool_dict
 
+    def check_hk_cbbc(self, code, start_date, end_date):
+
+        try:
+            code_length = len(code) > 0
+        except:
+            code_length = False
+        try:
+            start_date_is_int = isinstance(start_date, int)
+        except:
+            start_date_is_int = False
+        try:
+            start_date_length = len(str(start_date)) == 8
+        except:
+            start_date_length = False
+        try:
+            end_date_is_int = isinstance(end_date, int)
+        except:
+            end_date_is_int = False
+        try:
+            end_date_length = len(str(end_date)) == 8
+        except:
+            end_date_length = False
+
+
+        check_bool_dict = {'code_length': code_length,
+                           'start_date_length': start_date_length,
+                           'start_date_is_int': start_date_is_int,
+                           'end_date_is_int': end_date_is_int,
+                           'end_date_length': end_date_length,}
+
+        return check_bool_dict
+
+
     def check_hk_code_args(self, code):
 
         try:
